from __future__ import annotations

"""
Copyright (C) 2025 Luciano Guerche

This file is part of rabbitmq-mcp-server.

rabbitmq-mcp-server is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

rabbitmq-mcp-server is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with rabbitmq-mcp-server. If not, see <https://www.gnu.org/licenses/>.
"""

"""CLI entry point for rabbitmq-mcp-server."""

import click

from cli.commands.binding import binding
from cli.commands.exchange import exchange
from cli.commands.queue import queue


@click.group()
def cli() -> None:
    """Interact with RabbitMQ topology operations."""


cli.add_command(queue)
cli.add_command(exchange)
cli.add_command(binding)


__all__ = ["cli"]
